import requests
from magik.constants import API_BASE_URL
from magik.config import get_magik_api_key


# Log the request and response from OpenAI chat completion to Magik API
#
# prompt_slug: name of the prompt for analytics and grouping
# messages: "messages" json array sent to OpenAI chat completion
# model: string id of the language model used
# chat_completion: "choices" json response from OpenAI chat completion
def log_open_ai_chat_response(
    prompt_slug, messages, model, chat_completion, response_time, context
):
    """
    Track the request and response.
    """
    requests.post(
        f"{API_BASE_URL}/api/v1/savePromptAndResponse",
        json={
            "prompt_slug": prompt_slug,
            "prompt_data": messages,
            "language_model_id": model,
            "completion": chat_completion,
            "response_time": response_time,
            "context": context,
        },
        headers={
            "magik-api-key": get_magik_api_key(),
        },
    )



from magik.openai_helper import OpenAI

classifier_prompt = """
You are acting as a classifier. 
I want you to classify this output into one of the classification labels I am describing.
I am giving you the classification labels along with a description of the label.

Use the description to understand what each label means.
Then classify the output I am providing you as one of the labels, depending on which description matches most closely.

Your response should contain ONLY the classification label, and nothing else.
If no description matches, then return "None"

For example:
"""

classifier_prompt_example = """
Classification Labels and Descriptions: {[
    {"label": "plod", "description": "Represents a mammal."},
    {"label": "zazu", "description": "Represents a bird"},
    {"label": "goon", "description": "Represents a fish or sea animal"},
]}

Output to classify: "Eagles are majestic birds that fly high in the sky."
AI: "zazu"

Output to classify: "Sharks are dangerous and should be avoided."
AI: "goon"

Output to classify: "Lions are kings of the jungle."
AI: "plod"
"""


def classify_output(
    classification_labels_and_descriptions: list[dict],
    input_description: str,
    task_description: str,
    output_to_test=None,
):
    openai = OpenAI()
    response_message = openai.openai_chat_completion_message(
        model="gpt-3.5-turbo",
        prompt=f"""
            {classifier_prompt}
            {classifier_prompt_example}
            
            Now here is my data to classify:
            ---
            My input represents: {input_description}
            I want to determine: {task_description}
            Classification Labels and Descriptions: {classification_labels_and_descriptions}
            Output to classify: {output_to_test}
            
        """,
    )
    return response_message



import numpy as np
from magik.openai_helper import OpenAI


# Similarity score
def _cosine_similarity_from_embeddings(e1: list[float], e2: list[float]) -> float:
    # Convert the embedding lists to numpy arrays
    v1 = np.array(e1)
    v2 = np.array(e2)

    # Calculate the dot product of the two vectors
    dot_product = np.dot(v1, v2)

    # Calculate the magnitudes of the vectors
    magnitude_v1 = np.linalg.norm(v1)
    magnitude_v2 = np.linalg.norm(v2)

    # Calculate the cosine similarity
    similarity = dot_product / (magnitude_v1 * magnitude_v2)

    return similarity


def similarity_score(str1, str2, model):
    openai = OpenAI()
    e1 = openai.get_embedding(str1, model=model)
    e2 = openai.get_embedding(str2, model=model)
    score = _cosine_similarity_from_embeddings(e1, e2)
    return score



import os
from magik.internal_logger import logger
from magik.sys_exec import write_to_file
from magik.constants import (
    TEST_RUNS_DIR,
    TEST_DIR,
    CONFIG_FILE_PATH,
    SCHEDULE_CONFIG_FILE_PATH,
)


# Create magik_tests directory
# Create magik_test_runs directory
# Create magik_tests/magik_config.json
# Create magik_tests/schedule.json
def initialize():
    _create_magik_tests_dir()
    _create_magik_test_runs_dir()
    _create_magik_config_file()
    _create_schedule_config_file()


def _create_magik_tests_dir():
    if not os.path.exists(TEST_DIR):
        os.makedirs(TEST_DIR, exist_ok=True)
        logger.info(f"✅ Created {TEST_DIR} directory")
    else:
        logger.info(f"{TEST_DIR} directory already exists")


def _create_magik_test_runs_dir():
    if not os.path.exists(TEST_RUNS_DIR):
        os.makedirs(TEST_RUNS_DIR, exist_ok=True)
        logger.info(f"✅ Created {TEST_RUNS_DIR} directory")
    else:
        logger.info(f"{TEST_RUNS_DIR} directory already exists")


def _create_magik_config_file():
    # Check if magik_config.json already exists
    if os.path.exists(CONFIG_FILE_PATH):
        logger.info(f"{CONFIG_FILE_PATH} already exists!")
        return

    # Create file magik_config.json
    logger.debug("Creating magik_config.json...")
    example_config_contents = """{
    "MAGIK_API_KEY": "",
    "OPEN_AI_API_KEY": "",
    "OPEN_AI_DEFAULT_MODEL": "gpt-3.5-turbo"
}"""
    write_to_file(
        CONFIG_FILE_PATH,
        example_config_contents,
    )

    logger.info(f"✅ Created {CONFIG_FILE_PATH} (config file)")

    # add config file to gitignore
    _add_to_gitignore()

    logger.info("")
    logger.log_with_color(
        f"IMP: Please fill in the API keys in {CONFIG_FILE_PATH}", "yellow"
    )
    logger.info("")


def _create_schedule_config_file():
    # Check if magik_config.json already exists
    if os.path.exists(SCHEDULE_CONFIG_FILE_PATH):
        logger.info(f"{SCHEDULE_CONFIG_FILE_PATH} already exists!")
        return

    # Create file magik_config.json
    logger.debug("Creating schedule_config.json...")

    example_schedule_config_contents = """[
  {
    "trigger_type": "by-slug",
    "trigger_data": [
      {
        "prompt_slug": "*",
        "test_slugs": "*"
      }
    ]
  }
]"""
    write_to_file(
        SCHEDULE_CONFIG_FILE_PATH,
        example_schedule_config_contents,
    )

    logger.info(f"✅ Created {SCHEDULE_CONFIG_FILE_PATH} (schedule config)")
    logger.info("")


def _add_to_gitignore():
    gitignore_path = ".gitignore"
    if not os.path.exists(gitignore_path):
        logger.info(f"{gitignore_path} does not exist!")
        logger.info(
            f"Make sure to add {CONFIG_FILE_PATH} to your gitignore file")
        return

    with open(gitignore_path, "a") as gitignore_file:
        gitignore_file.write(
            "\n# Magik config file\nmagik_config.json\nmagik_tests/test_runs/*\n"
        )

    logger.info(f"✅ Added {CONFIG_FILE_PATH} to {gitignore_path}")
